@@ -126,6 +126,31 @@ setClass("git_signature",
          }
 )
 
+##' Class \code{"cred_env"}
+##'
+##' @title S4 class to handle environmental variables credential
+##' object
+##' @section Slots:
+##' \describe{
+##'   \item{username}{
+##'     The name of the environmental variable that holds
+##'     the username for the authentication.
+##'   }
+##'   \item{password}{
+##'     The name of the environmental variable that holds
+##'     the password for the authentication.
+##'   }
+##' }
+##' @rdname cred_env-class
+##' @docType class
+##' @keywords classes
+##' @keywords methods
+##' @export
+setClass("cred_env",
+         slots = c(username = "character",
+                   password = "character")
+)
+
 ##' Class \code{"cred_user_pass"}
 ##'
 ##' @title S4 class to handle plain-text username and password
@@ -394,8 +419,8 @@ setClass("git_diff",
          slots = c(old   = "ANY",
                    new   = "ANY",
                    files = "list"),
-         prototype = list(old=NA_character_,
-                          new=NA_character_))
+         prototype = list(old = NA_character_,
+                          new = NA_character_))
 
 ##' Git diff file
 ##'
@@ -559,8 +584,8 @@ setClass("git_reference",
                    sha       = "character",
                    target    = "character",
                    shorthand = "character"),
-         prototype=list(sha    = NA_character_,
-                        target = NA_character_))
+         prototype = list(sha    = NA_character_,
+                          target = NA_character_))
 
 ##' Class \code{"git_reflog_entry"}
 ##'
@@ -645,7 +670,7 @@ setClass("git_tag",
                    tagger  = "git_signature",
                    target  = "character",
                    repo    = "git_repository"),
-         validity=function(object)
+         validity = function(object)
          {
              errors <- validObject(object@tagger)
 
@@ -701,7 +726,7 @@ setClass("git_tree",
                    id       = "character",
                    name     = "character",
                    repo     = "git_repository"),
-         validity=function(object)
+         validity = function(object)
          {
              errors <- character(0)
 
