@@ -20,11 +20,16 @@
 ##' area.
 ##' @rdname status-methods
 ##' @docType methods
-##' @param repo The \code{git_repository} to get status from.
+##' @param repo The repository \code{object}
+##' \code{\linkS4class{git_repository}} to get status from. If the
+##' \code{repo} argument is missing, the repository is searched for
+##' with \code{\link{discover_repository}} in the current working
+##' directory.
 ##' @param staged Include staged files. Default TRUE.
 ##' @param unstaged Include unstaged files. Default TRUE.
 ##' @param untracked Include untracked files. Default TRUE.
 ##' @param ignored Include ignored files. Default FALSE.
+##' @param ... Additional arguments to status.
 ##' @return S3 class \code{git_status} with repository status
 ##' @keywords methods
 ##' @include S4_classes.r
@@ -76,20 +81,22 @@ setGeneric("status",
                     staged    = TRUE,
                     unstaged  = TRUE,
                     untracked = TRUE,
-                    ignored   = FALSE)
+                    ignored   = FALSE,
+                    ...)
            standardGeneric("status"))
 
 ##' @rdname status-methods
 ##' @export
 setMethod("status",
           signature(repo = "missing"),
-          function(repo, staged, unstaged, untracked, ignored)
+          function(staged, unstaged, untracked, ignored, ...)
           {
-              status(repo      = repository(getwd(), discover = TRUE),
-                     staged    = staged,
-                     unstaged  = unstaged,
-                     untracked = untracked,
-                     ignored   = ignored)
+              callGeneric(repo      = lookup_repository(),
+                          staged    = staged,
+                          unstaged  = unstaged,
+                          untracked = untracked,
+                          ignored   = ignored,
+                          ...)
           }
 )
 
@@ -97,7 +104,7 @@ setMethod("status",
 ##' @export
 setMethod("status",
           signature(repo = "git_repository"),
-          function(repo, staged, unstaged, untracked, ignored)
+          function(repo, staged, unstaged, untracked, ignored, ...)
           {
               structure(.Call(git2r_status_list, repo, staged,
                               unstaged, untracked, ignored),
