@@ -81,16 +81,18 @@ static const R_CallMethodDef callMethods[] =
     {"git2r_branch_target", (DL_FUNC)&git2r_branch_target, 1},
     {"git2r_branch_upstream_canonical_name", (DL_FUNC)&git2r_branch_upstream_canonical_name, 1},
     {"git2r_checkout_tree", (DL_FUNC)&git2r_checkout_tree, 3},
-    {"git2r_clone", (DL_FUNC)&git2r_clone, 5},
+    {"git2r_clone", (DL_FUNC)&git2r_clone, 6},
     {"git2r_commit", (DL_FUNC)&git2r_commit, 4},
     {"git2r_commit_parent_list", (DL_FUNC)&git2r_commit_parent_list, 1},
     {"git2r_commit_tree", (DL_FUNC)&git2r_commit_tree, 1},
     {"git2r_config_get", (DL_FUNC)&git2r_config_get, 1},
+    {"git2r_config_get_logical", (DL_FUNC)&git2r_config_get_logical, 2},
+    {"git2r_config_get_string", (DL_FUNC)&git2r_config_get_string, 2},
     {"git2r_config_set", (DL_FUNC)&git2r_config_set, 2},
     {"git2r_diff", (DL_FUNC)&git2r_diff, 5},
     {"git2r_graph_ahead_behind", (DL_FUNC)&git2r_graph_ahead_behind, 2},
     {"git2r_graph_descendant_of", (DL_FUNC)&git2r_graph_descendant_of, 2},
-    {"git2r_index_add_all", (DL_FUNC)&git2r_index_add_all, 2},
+    {"git2r_index_add_all", (DL_FUNC)&git2r_index_add_all, 3},
     {"git2r_index_remove_bypath", (DL_FUNC)&git2r_index_remove_bypath, 2},
     {"git2r_libgit2_features", (DL_FUNC)&git2r_libgit2_features, 0},
     {"git2r_libgit2_version", (DL_FUNC)&git2r_libgit2_version, 0},
@@ -107,6 +109,7 @@ static const R_CallMethodDef callMethods[] =
     {"git2r_odb_hashfile", (DL_FUNC)&git2r_odb_hashfile, 1},
     {"git2r_odb_objects", (DL_FUNC)&git2r_odb_objects, 1},
     {"git2r_push", (DL_FUNC)&git2r_push, 4},
+    {"git2r_reference_dwim", (DL_FUNC)&git2r_reference_dwim, 2},
     {"git2r_reference_list", (DL_FUNC)&git2r_reference_list, 1},
     {"git2r_reflog_list", (DL_FUNC)&git2r_reflog_list, 2},
     {"git2r_remote_add", (DL_FUNC)&git2r_remote_add, 3},
