@@ -27,8 +27,7 @@ SEXP git2r_remote_fetch(
     SEXP repo,
     SEXP name,
     SEXP credentials,
-    SEXP msg,
-    SEXP who);
+    SEXP msg);
 SEXP git2r_remote_list(SEXP repo);
 SEXP git2r_remote_remove(SEXP repo, SEXP remote);
 SEXP git2r_remote_rename(SEXP repo, SEXP oldname, SEXP newname);
