@@ -75,17 +75,17 @@ SEXP git2r_revwalk_list(
     git_repository *repository = NULL;
 
     if (git2r_arg_check_logical(topological))
-        git2r_error(git2r_err_logical_arg, __func__, "topological");
+        git2r_error(__func__, NULL, "'topological'", git2r_err_logical_arg);
     if (git2r_arg_check_logical(time))
-        git2r_error(git2r_err_logical_arg, __func__, "time");
+        git2r_error(__func__, NULL, "'time'", git2r_err_logical_arg);
     if (git2r_arg_check_logical(reverse))
-        git2r_error(git2r_err_logical_arg, __func__, "reverse");
+        git2r_error(__func__, NULL, "'reverse'", git2r_err_logical_arg);
     if (git2r_arg_check_integer(max_n))
-        git2r_error(git2r_err_integer_arg, __func__, "max_n");
+        git2r_error(__func__, NULL, "'max_n'", git2r_err_integer_arg);
 
     repository = git2r_repository_open(repo);
     if (!repository)
-        git2r_error(git2r_err_invalid_repository, __func__, NULL);
+        git2r_error(__func__, NULL, git2r_err_invalid_repository, NULL);
 
     if (git_repository_is_empty(repository)) {
         /* No commits, create empty list */
@@ -101,11 +101,11 @@ SEXP git2r_revwalk_list(
         sort_mode |= GIT_SORT_REVERSE;
 
     err = git_revwalk_new(&walker, repository);
-    if (GIT_OK != err)
+    if (err)
         goto cleanup;
 
     err = git_revwalk_push_head(walker);
-    if (GIT_OK != err)
+    if (err)
         goto cleanup;
     git_revwalk_sorting(walker, sort_mode);
 
@@ -117,7 +117,7 @@ SEXP git2r_revwalk_list(
 
     git_revwalk_reset(walker);
     err = git_revwalk_push_head(walker);
-    if (GIT_OK != err)
+    if (err)
         goto cleanup;
     git_revwalk_sorting(walker, sort_mode);
 
@@ -127,14 +127,14 @@ SEXP git2r_revwalk_list(
         git_oid oid;
 
         err = git_revwalk_next(&oid, walker);
-        if (GIT_OK != err) {
+        if (err) {
             if (GIT_ITEROVER == err)
                 err = GIT_OK;
             goto cleanup;
         }
 
         err = git_commit_lookup(&commit, repository, &oid);
-        if (GIT_OK != err)
+        if (err)
             goto cleanup;
 
         SET_VECTOR_ELT(result, i, item = NEW_OBJECT(MAKE_CLASS("git_commit")));
@@ -152,8 +152,8 @@ cleanup:
     if (R_NilValue != result)
         UNPROTECT(1);
 
-    if (GIT_OK != err)
-        git2r_error(git2r_err_from_libgit2, __func__, giterr_last()->message);
+    if (err)
+        git2r_error(__func__, giterr_last(), NULL, NULL);
 
     return result;
 }
@@ -187,15 +187,15 @@ SEXP git2r_revwalk_contributions(
     git_repository *repository = NULL;
 
     if (git2r_arg_check_logical(topological))
-        git2r_error(git2r_err_logical_arg, __func__, "topological");
+        git2r_error(__func__, NULL, "'topological'", git2r_err_logical_arg);
     if (git2r_arg_check_logical(time))
-        git2r_error(git2r_err_logical_arg, __func__, "time");
+        git2r_error(__func__, NULL, "'time'", git2r_err_logical_arg);
     if (git2r_arg_check_logical(reverse))
-        git2r_error(git2r_err_logical_arg, __func__, "reverse");
+        git2r_error(__func__, NULL, "'reverse'", git2r_err_logical_arg);
 
     repository = git2r_repository_open(repo);
     if (!repository)
-        git2r_error(git2r_err_invalid_repository, __func__, NULL);
+        git2r_error(__func__, NULL, git2r_err_invalid_repository, NULL);
 
     if (git_repository_is_empty(repository))
         goto cleanup;
@@ -208,11 +208,11 @@ SEXP git2r_revwalk_contributions(
         sort_mode |= GIT_SORT_REVERSE;
 
     err = git_revwalk_new(&walker, repository);
-    if (GIT_OK != err)
+    if (err)
         goto cleanup;
 
     err = git_revwalk_push_head(walker);
-    if (GIT_OK != err)
+    if (err)
         goto cleanup;
     git_revwalk_sorting(walker, sort_mode);
 
@@ -231,7 +231,7 @@ SEXP git2r_revwalk_contributions(
 
     git_revwalk_reset(walker);
     err = git_revwalk_push_head(walker);
-    if (GIT_OK != err)
+    if (err)
         goto cleanup;
     git_revwalk_sorting(walker, sort_mode);
 
@@ -241,14 +241,14 @@ SEXP git2r_revwalk_contributions(
         git_oid oid;
 
         err = git_revwalk_next(&oid, walker);
-        if (GIT_OK != err) {
+        if (err) {
             if (GIT_ITEROVER == err)
                 err = GIT_OK;
             goto cleanup;
         }
 
         err = git_commit_lookup(&commit, repository, &oid);
-        if (GIT_OK != err)
+        if (err)
             goto cleanup;
 
         c_author = git_commit_author(commit);
@@ -270,8 +270,8 @@ cleanup:
     if (R_NilValue != result)
         UNPROTECT(1);
 
-    if (GIT_OK != err)
-        git2r_error(git2r_err_from_libgit2, __func__, giterr_last()->message);
+    if (err)
+        git2r_error(__func__, giterr_last(), NULL, NULL);
 
     return result;
 }
