@@ -22,7 +22,7 @@ dir.create(path)
 
 ## Initialize a repository
 repo <- init(path)
-config(repo, user.name="Developer", user.email="developer@example.org")
+config(repo, user.name="Alice", user.email="alice@example.org")
 
 ## Create a file and commit
 writeLines("Hello world!", file.path(path, "test.txt"))
