@@ -110,8 +110,8 @@ stopifnot(identical(m_3@sha, character(0)))
 stopifnot(identical(status(repo),
                     structure(list(staged = structure(list(),
                                        .Names = character(0)),
-                                   unstaged = structure(list(unmerged = "test.txt"),
-                                       .Names = "unmerged"),
+                                   unstaged = structure(list(conflicted = "test.txt"),
+                                       .Names = "conflicted"),
                                    untracked = structure(list(),
                                        .Names = character(0))),
                               .Names = c("staged", "unstaged", "untracked"),
